#!/usr/local/bin/perl

# to be ran nightly (or more) to expire requests that have been
# waiting too long or revoked

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use Net::SMTP;
use IPAnalyze::Misc;
use IPAnalyze::IPmath;
use IPAnalyze::Customers;

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open config: $!\n";
while(<CONFIG>){
   next if(/^[#;]/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $d_dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
my $c_dbh = $d_dbh;
if($cfg{c_dsn} && $cfg{c_dbun} && $cfg{c_dbpw}){
   unless(($cfg{d_dsn} eq $cfg{c_dsn}) && ($cfg{d_dbun} eq $cfg{c_dbun}) && ($cfg{d_dbpw} eq $cfg{c_dbpw})){
      $c_dbh = DBI->connect($cfg{c_dsn}, $cfg{c_dbun}, $cfg{c_dbpw}, {RaiseError => 1});
   }
}

my $misc = IPAnalyze::Misc->new();
my $ipmath = IPAnalyze::IPmath->new();
my $customers = IPAnalyze::Customers->new();

my $our_name = lc($cfg{our_name});
$our_name =~ s/\s//g;
my $ucour_name = uc($our_name);

my $date = localtime();
my $now = $^T;
my %seconds = (30 => '2592000', 55 => '4752000', 60 => '5184000', 85 => '7344000', 90 => '6912000'); # key is days

my $sendinfo;
my $revoke = ($now - $seconds{30});
my $sql = 'SELECT netid,netblock,cidr FROM data WHERE status = 8 AND last_updated <= ' . $d_dbh->quote($revoke);
my $sth = $d_dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	my $code = $misc->check_igp($ipmath,$cfg{'igp_host'},$cfg{'igp_community'},$row->[1],$row->[2]);
	if($code){
		warn "revoking $row->[0] - $row->[1]/$row->[2]\n";
   	my $sqla = 'DELETE FROM data WHERE netid = ' . $d_dbh->quote($row->[0]);
   	my $stha = $d_dbh->prepare($sqla);
   	$stha->execute;
	}else{
		$sendinfo=1;
		warn "$row->[0] - $row->[1]/$row->[2] is in IGP, but IPAnalyze says it's REVOKED ($code)\n";
	}
}

my $delta = ($now - $seconds{55}); # go back 55 days, and look at everything older
# expire requests waiting too long
my $stuff = 'netid,circuit_id,netblock,cidr,local_email,tech_email,bornondate,status,cust_id';
$sql = "SELECT $stuff FROM data WHERE status != 0 AND bornondate < $delta ORDER BY bornondate";
$sth = $d_dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	my $lt = localtime($row->[6]);

	my $oc_status = $customers->status($c_dbh,$d_dbh,$row->[1],$row->[8]);

	my $seconds_in_system = ($now - $row->[6]);
	my $key = ($oc_status eq 'Working' || $oc_status eq 'Cancelled') ? 60 : 90;
	my	$daysleft = sprintf('%.0f',(($seconds{$key} - $seconds_in_system)/86400));

	next unless($daysleft == 5 || $daysleft <= 1);
	my	$pending = ($seconds_in_system < $seconds{$key}) ? 1 : 0;

	warn "checking netid $row->[0] netblock $row->[2]\n";
	unless($pending){
		my $code = 1; # if netblock is not allocated yet (sitting in allocate, etc)
		if($row->[2]){
			# if netblock is allocated
			$code = $misc->check_igp($ipmath,$cfg{'igp_host'},$cfg{'igp_community'},$row->[2],$row->[3]);
		}
		if($code){
			my $sqla = 'DELETE FROM data WHERE netid = ' . $d_dbh->quote($row->[0]);
			#my $stha = $d_dbh->prepare($sqla);
			#$stha->execute;
			warn "$sqla: $sqla\n";
		}else{
			$sendinfo=1;
			warn "$row->[0] - $row->[2]/$row->[3] is in IGP, but IPAnalyze says it's NOT ACTIVE\n";
		}
	}

	if($row->[7] < 6){
		warn "sending mail\n";
		my $smtp = Net::SMTP->new($cfg{'mailhost'});
		$smtp->mail('ipanalyze@ipanalyze.example.com');
		$smtp->recipient($row->[4],$row->[5],'isp@example.com', {SkipBad => 1});
		$smtp->data();
		$smtp->datasend("To: \"$cfg{our_name} Customer\"\n");
		$smtp->datasend("From: \"$cfg{our_name} - IPAnalyze\" <>\n");
		if($pending){
			$smtp->datasend("Subject: Pending Expiration of network for $row->[1]\n");
		}else{
			$smtp->datasend("Subject: Expiration of network for $row->[1]\n");
		}
		$smtp->datasend("Date: $date\n");
		$smtp->datasend("\n");
		$smtp->datasend("Hello,\nThis email is to inform you that the network\n");
		$smtp->datasend("$row->[2]/$row->[3] for Circuit ID $row->[1]\n");
		if($pending){
			$smtp->datasend("is about to expire.  If this network is still required, it must be\n");
			$smtp->datasend("assigned within ${daysleft} days.  If left unassigned, this network will expire.\n");
		}else{
			$smtp->datasend("has expired.\n");
		}
		$smtp->datasend("\nPlease call $cfg{our_phone} or write to $cfg{support_email} if you have any questions.");
		$smtp->datasend();
		$smtp->quit;
	}
}

if($sendinfo){
	warn "\n\n",
	     "Open IPAnalyze, search for each NetID, and check the routers for a matching route.\n",
	     "If a route matches, trace down where it's going, and make sure its going to the customer\n",
	     "that IPAnalyze thinks it should be going to -- you might also need to check radius.\n",
	     "If no route matches, investigate further.\n";
}
