#!/usr/local/bin/perl
#IP Analyze 

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use warnings;
use HTML::Template;
use CGI qw(:standard -nosticky);
use CGI::Carp qw(fatalsToBrowser);
use Sys::SigAction qw(set_sig_handler);
use DBI;
use Net::SMTP;
use Mail::VRFY;
use IPAnalyze::Auth;
use IPAnalyze::Misc;
use IPAnalyze::IPmath;
use IPAnalyze::Customers;

our $ipmanage = 1;
our $ipmonitor = 1;
our $script = $ENV{'SCRIPT_NAME'};
our $login = 'http://' . $ENV{'HTTP_HOST'} . '/cgi-bin/login.mpl';

our %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open ipanalyze.conf: $!\n";
while(<CONFIG>){
	s/^(.*)\s*#.*/$1/;
	next if(/^$/);
	chomp;
	if(/^(\S+)\s*=>\s?(.+)/){
		$cfg{$1} = $2;
	}
}
close CONFIG;

our %status = (0=>'Complete',
               1=>'Waiting on network diagram',
               2=>'Waiting on allocation',
               3=>'Waiting on assignment',
               4=>'Holding',
               5=>'Denied',
               6=>'Hold Pending',
               7=>'Revoked');

our %color = (Closed => '#FF0000',
              Cancelled => '#FF0000',
              'Pending Close' => '#FFFF00',
             );
              

my $d_dbh;
until($d_dbh){
	eval {
		my $h = set_sig_handler('ALRM', sub { die "DBConnect timeout (d_dbh)."; });
		alarm(10);
		$d_dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});
		alarm(0);
	};
	alarm(0);
	warn "$DBI::errstr\n" if($DBI::errstr);
	if($@){
		warn "error: $@\n";
	}
	sleep 1;
}
our $c_dbh = $d_dbh;
if($cfg{c_dsn} && $cfg{c_dbun} && $cfg{c_dbpw}){
	unless(($cfg{d_dsn} eq $cfg{c_dsn}) && ($cfg{d_dbun} eq $cfg{c_dbun}) && ($cfg{d_dbpw} eq $cfg{c_dbpw})){
		$c_dbh = undef;
		until($c_dbh){
			eval {
				my $h = set_sig_handler('ALRM', sub { die "DBConnect timeout (c_dbh)."; });
				alarm(10);
				$c_dbh = DBI->connect ($cfg{c_dsn}, $cfg{c_dbun}, $cfg{c_dbpw}, {RaiseError => 1});
				alarm(0);
			};
			alarm(0);
			warn "$DBI::errstr\n" if($DBI::errstr);
			if($@){
				warn "Error: $@\n";
			}
			sleep 1;
		}
	}
}

my $q = CGI->new();
my $auth = IPAnalyze::Auth->new();
our $misc = IPAnalyze::Misc->new();
our $ipmath = IPAnalyze::IPmath->new();
our $customers = IPAnalyze::Customers->new();

my ($username,$access,@error) = $auth->check_access($d_dbh,$q,$cfg{app});
unless($username){
	my $redirect = $login . '?dest=' . $script;
	print $q->redirect($redirect);
	exit;
}
print $q->header(-expires=>'now'); # GLOBAL
if(@error){
	foreach my $error (@error){
		print "<p align=\"center\"><i><font color=\"#FF0000\"><b>$error</b></font></i></p>";
	}
	exit;
}
our %areas = $misc->get_areas($d_dbh);

if(! $q->param){
	show_main($d_dbh,$username,$q,'default','','',1,$access);
	exit;
}elsif($q->param('search')){
	my (%query,$t);
	my $cn = $q->param('company_name');
	my $sql = 'SELECT netid,cust_id,circuit_id,status,netblock,cidr FROM data';
	if($cn){
		my $cust_ids = $customers->cust_ids($c_dbh,'name',$cn); # arrayref
		if($cust_ids){
			$t = 1;
			if(my $num = @{$cust_ids}){
				$sql .= ' WHERE (';
				foreach my $id (@{$cust_ids}){
					$num--;
					$sql .= ' cust_id = ' . $d_dbh->quote($id);
					$sql .= ' OR' if($num);
				}
				$sql .= ')';
			}else{
				$sql .= ' WHERE 0';
			}
		}else{
			warn "cust_ids is blank\n";
		}
	}
	unless($q->param('search') eq 'show_all'){
		if($q->param('netid') ne ''){
			$query{netid} = $q->param('netid');
		}
		if($q->param('network') =~ /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\/(\d{1,2})$/){
			$query{netblock} = $1;
			$query{cidr} = $2;
		}elsif($q->param('network') ne ''){
			$query{netblock} = $q->param('network');
		}
		if ($q->param('cust_id') ne ''){
			$query{cust_id} = $q->param('cust_id');
		}
		if ($q->param('circuit_id') ne ''){
			$query{circuit_id} = $q->param('circuit_id');
			$query{circuit_id} =~ s/^\s*//;
			$query{circuit_id} =~ s/\s*$//;
		}
		while (my($key,$value) = each %query){
			if($t){
				$sql .= " AND $key LIKE " . $d_dbh->quote("%${value}%");
			}else{
				$t = 1;
				$sql .= " WHERE $key LIKE " . $d_dbh->quote("%${value}%");
			}
		}
		if ($q->param('status') ne ''){
			$sql .= ($t) ? ' AND' : ' WHERE';
			$sql .= ' status = ' . $d_dbh->quote($q->param('status'));
			$t = 1;
		}
		unless($t || $cn){
			print $q->start_html("IP Analyze v$cfg{version}"),
			      "Please enter search criteria before pressing Search button<br>\n",
			      $q->end_html();
			exit;
		}
	}
	my $sth = $d_dbh->prepare($sql);
	$sth->execute();
	my(@request_data,$res);
	my $template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
	$template->param(VIEW_SEARCH_RESULT => 1);
	my (%cname,%cstatus);
	while (my $data = $sth->fetchrow_arrayref){
		if($cname{$data->[1]} && (! $cstatus{$data->[2]})){
			$cstatus{$data->[2]} = $customers->status($c_dbh,$d_dbh,$data->[2],$data->[1]);
		}elsif($cstatus{$data->[2]} && (! $cname{$data->[1]})){
			$cname{$data->[1]} = $customers->name($c_dbh,$d_dbh,'cust_id',$data->[1]);
		}elsif((! $cname{$data->[1]}) && (! $cstatus{$data->[2]})){
			my $all = $customers->all($c_dbh,$d_dbh,'circuit_id',$data->[2]);
			$cname{$data->[1]} = $all->[1];
			$cstatus{$data->[2]} = $all->[6];
		}

		my $network = $data->[4] . '/' . $data->[5];
		push @request_data, {
		  NETWORK => $network,
		  BGCOLOR => $color{$cstatus{$data->[2]}} || '#FFFFFF',
		  CUST_ID => $data->[1],
		  NETID => $data->[0],
		  CIRCUIT_ID => $data->[2],
		  STATUS => $status{$data->[3]},
		  OC8_STATUS => $cstatus{$data->[2]},
		  COMPANY_NAME => $cname{$data->[1]},
		};
		$res++;
	}
	if($res){
		my @sorted = sort { $a->{COMPANY_NAME} cmp $b->{COMPANY_NAME} ||
	   	                 $a->{CIRCUIT_ID} cmp $b->{CIRCUIT_ID} } @request_data;
		$template->param(REQUEST_DATA => \@sorted);
	}else{
		$template->param(NO_RESULTS_FOUND => 1);
	}
	$template->param(SCRIPT => $script);
	$template->param(IPMANAGE => $ipmanage);
	$template->param(IPMONITOR => $ipmonitor);
	$template->param(VERSION => $cfg{version});
	$template->param(USERNAME => $username);
	print $template->output;

}elsif(defined(my $func = $q->param('action'))){

	if ($func eq 'show_main'){
		show_main($d_dbh,$username,$q,'default','','',1,$access);
	}elsif ($func eq 'search'){
		show_main($d_dbh,$username,$q,'show_search','null','search',1,$access);
	}elsif ($func eq 'view_details'){
		my $cust_id = $q->param('cust_id');
		my $atype = $q->param('aty');
		show_main($d_dbh,$username,$q,'view_details',$cust_id,$atype,1,$access);
	}elsif($func eq 'view_usage'){
		my $netid = $q->param('netid');
		my $atype = $q->param('aty');
		show_main($d_dbh,$username,$q,'view_usage',$netid,$atype,1,$access);
	}elsif($func eq 'view_allocation'){
		show_main($d_dbh,$username,$q,'view_allocation','','',1,$access);
	}elsif($func eq 'view_cidr'){
		show_main($d_dbh,$username,$q,'view_cidr','','',1,$access);
	}elsif ($func == 1){
		show_main($d_dbh,$username,$q,'view_list_data',1,'and',1,$access);
	}elsif ($func == 2){
		show_main($d_dbh,$username,$q,'view_list_data',2,'allocate',1,$access);
	}elsif ($func == 3){
		show_main($d_dbh,$username,$q,'view_list_data',3,'assign',1,$access);
	}elsif ($func == 4){
		show_main($d_dbh,$username,$q,'view_list_data',4,'holding',1,$access);
	}elsif ($func == 6){
		show_main($d_dbh,$username,$q,'view_list_data',6,'hold_pending',1,$access);
	}elsif($func == 7){
		show_main($d_dbh,$username,$q,'view_list_data',7,'revoked',1,$access);
	}else{
		die "not implemented: $func";
	}
	exit;

}elsif(defined(my $netid=$q->param('view_network'))){
	die "bad view_network: $netid\n" unless($netid == int($netid));
	my $atype = $q->param('aty');
	show_main($d_dbh,$username,$q,'view_network',$netid,$atype,1,$access);
}elsif(defined($q->param('edit'))){
	my $edit_type = $q->param('editw');
	my $netid = $q->param('netid');
	my $atype = $q->param('aty');
	if ($edit_type eq 'contact_info'){
		my $cust_id = $q->param('cust_id');
		show_main($d_dbh,$username,$q,'edit_contact',$cust_id,$atype,1,$access);
		exit;
	}elsif ($edit_type eq 'in_contact_info'){
		show_main($d_dbh,$username,$q,'edit_in_contact',$netid,$atype,1,$access);
	}elsif ($edit_type eq 'tech_contact_info'){
		show_main($d_dbh,$username,$q,'edit_tech_contact',$netid,$atype,1,$access);
	}elsif ($edit_type eq 'request_info'){
		show_main($d_dbh,$username,$q,'edit_request',$netid,$atype,3,$access);
	}elsif ($edit_type eq 'monitoring_info'){
		show_main($d_dbh,$username,$q,'edit_monitoring',$netid,$atype,3,$access);
	}
}elsif(my $update=$q->param('update')){
	my ($sql,$link);
	my $mail_template = "$cfg{home_dir}/data/templates/ipanalyze/text/";
	my $netid = $q->param('netid');
	my $cust_id = $q->param('cust_id');
	my $new_acct = $q->param('new_acct');
	die "no cust or net id found\n" unless($netid || $cust_id);
	my $update_type = $q->param('update_type');
	my $aty = $q->param('aty');
	if ($update_type eq 'contact'){
		if ($access > 1){
			my %data = (company_name => $q->param('company_name'),
		 	            address      => $q->param('address'),
			            city         => $q->param('city'),
			            state        => $q->param('state'),
			            zip          => $q->param('zip'),
			            country      => $q->param('country'),
			            cust_id      => $new_acct,
			           );
			if($data{'company_name'} !~ /^[A-Z0-9\&\s\.-]+$/i){
				push @error , "Invalid Characters used for Company Name";
			}
			unless($data{'address'} =~ /^[A-Z0-9\s\,\.-]+$/i){
				push @error , "Invalid Characters used for Address";
			}
			unless($data{'city'} =~ /^[A-Z0-9\s\.-]+$/i){
				push @error , "Invalid Characters used for city";
			}
			unless($data{'state'} =~ /^[A-Z]{2}$/){
				push @error , "Invalid Characters used for state";
			}
			unless($data{'zip'} =~ /^\d{5}$/){
				push @error , "Invalid Zip Code: must be 5 numbers";
			}
			unless($data{'country'} =~ /^[A-Z]{2}$/ ){
				push @error , "Invalid Country code: must be 2 letter ISO 3166-1 code ";
			}
			unless($cust_id =~ /^\d{10}$/){
				push @error, "Invalid Characters used for Original Account Number: $cust_id";
			}
			unless($new_acct =~ /^\d{10}$/){
				push @error, "Invalid Characters used for New Account Number: $new_acct";
			}

			unless(@error){
				$customers->update($c_dbh,$cust_id,\%data);
			}

			$sql = 'UPDATE data SET cust_id = ' . $d_dbh->quote($new_acct);
			$sql .= ' WHERE cust_id = ' . $d_dbh->quote($cust_id);

			$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?action=view_details&cust_id=${new_acct}";
		} else {
			push @error, "You do not have sufficient permissions to perform this action (contact)";
		}
	}elsif($update_type eq 'in_contact'){
		if($access > 1){
			my $in_contact = $q->param('local_contact');
			my $in_email = $q->param('local_email');
			if ($in_contact !~ /^[A-Z0-9\s\.-]+$/i){
				push @error , "Invalid Characters used for Contact Name";
			}
			my $emailchk = Mail::VRFY::CheckAddress($in_email);
			if($emailchk){
				push @error, "You must enter a valid Contact E-mail";
			}

			$sql = 'UPDATE data SET local_contact = ' . $d_dbh->quote($in_contact);
			$sql .= ', local_email = ' . $d_dbh->quote($in_email);
			$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

			$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
		}else{
			push @error, "You do not have sufficient permissions to perform this action (in_contact)";
		}
	}elsif($update_type eq 'tech_contact'){
		if ($access > 1){
			my $techname = $q->param('tech_name');
			my $techemail = $q->param('tech_email');
			my $techphone = $q->param('tech_phone');
			my $techext = $q->param('tech_ext');
			my $private = $q->param('private');
			my $remove_holding = $q->param('remove_holding');
			
			unless(defined($private) && ($private == 0 || $private == 1)){
				push @error, "Private value incorrect: $private\n";
			}
			if ($techname !~ /^[A-Z0-9\s\.-]+$/i){
				push @error , "Invalid Characters used for Technical Contact Name";
			}
			my $emailchk = Mail::VRFY::CheckAddress($techemail);
			if ($emailchk){
				push @error, "You must enter a valid Technical Contact E-mail";
			}
			$techphone =~ s/-//g;
			unless($techphone =~ /^\d{10}$/){
				push @error, "Invalid Characters used for Technical Phone Number";
			}
			if(defined($techext)){
				unless(($techext =~ /^\d{1,10}$/) && ($techext <= 2147483647)){
					push @error, "Invalid extension";
				}
			}

			if ($remove_holding eq 'on'){
				my $timespan = (time() + 86400);
				my $sql = 'UPDATE data SET ';
				$sql .= 'status = 6, hold_date = ' . $d_dbh->quote($timespan);
				$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
				my $sth = $d_dbh->prepare($sql);
				$sth->execute();

				#get status before bounce so we can remember it
				my $check = 'SELECT ostatus FROM data WHERE netid = ' . $d_dbh->quote($netid);
				$sth = $d_dbh->prepare($check);
				$sth->execute();
				my $data = $sth->fetchrow_arrayref;
				###
				if($data->[0] == 1){
					$mail_template .= 'request_s1.txt';
				}elsif ($data->[0] == 2){
					$mail_template .= 'request_approve.txt';
				}elsif ($data->[0] == 3){
					$mail_template .= 'request_complete.txt';
				}else{
					warn "unkown ostatus code\n";
					$mail_template = undef;
				}
				$sth->finish;
			}

			$sql = 'UPDATE data SET tech_name = ' . $d_dbh->quote($techname);
			$sql .= ',tech_email = ' . $d_dbh->quote($techemail);
			$sql .= ',tech_phone = ' . $d_dbh->quote($techphone);
			if(defined($techext)){
				$sql .= ',tech_ext = ' . $d_dbh->quote($techext);
			}
			$sql .= ',private_id = ';
			if($private == 1){
				my $private_id = $misc->get_rand(32);
				$sql .= $d_dbh->quote($private_id);
			}else{
				$sql .= 'NULL';
			}
			$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

			$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
		} else {
			push @error, "You do not have sufficient permissions to perform this action (tech_contact)";
		}
	}elsif($update_type eq 'monitoring_info'){
		my $ts = localtime;
		if($access >= 5){
			$sql = 'SELECT notes FROM data WHERE netid = ' . $d_dbh->quote($netid);
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $row = $sth->fetchrow_arrayref;
			my $sqlnotes = $row->[0];

			$sql = 'UPDATE data SET ';
			my $change;
			if($q->param('MONITORING_UPDATE') && (my $enabled=$q->param('enabled'))){
				my $value = ($enabled eq 'YES') ? 1 : 0;
				$sql .= "enable_monitoring = $value";
				$change='monitoring';
			}elsif($q->param('EMAIL_UPDATE') && (my $warnings=$q->param('warnings'))){
				my $value = ($warnings eq 'YES') ? 1 : 0;
				$sql .= "enable_emailwar = $value";
				$change='warnings';
			}else{
				die "confuzzled";
			}
			$sql .= ',notes = ';
			$sql .= $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} changed ${change}");
			$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

			$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}"
		}else{
			push @error, "You do not have sufficient permissions to perform this action (monitoring_info)";
		}
	}elsif($update_type eq 'request_info'){
		my $notes = $q->param('notes');
		my $ts = localtime;
		if(($update eq 'Add Note') &&
		   (defined(my $newnote=$q->param('notes')))){
			my $sqlnotes = $misc->get_notes($d_dbh,$netid);

			$sql = 'UPDATE data SET notes = ';
			$sql .= $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username}: ${newnote}");
			$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
			$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";

		}elsif(($update eq 'Change CUST_ID') &&
		   (defined(my $new_cust_id=$q->param('new_cust_id')))){

				if($cust_id == int($cust_id)){
					my $count = $customers->count($c_dbh,$new_cust_id);
					if($count){
						my $sqlnotes = $misc->get_notes($d_dbh,$netid);

						$sql = 'UPDATE data SET cust_id = ' . $d_dbh->quote($new_cust_id);
						$sql .= ',notes = ' . $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} changed cust_id to ${new_cust_id}");
						$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
						$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
					}else{
						push @error, "cust_id $new_cust_id not found in OC8\n";
					}
				}else{
					push @error, "Invalid looking cust_id\n";
				}
		}elsif(($update eq 'Change Use') &&
		       (defined(my $use=$q->param('network_use')))){
				my $sqlnotes = $misc->get_notes($d_dbh,$netid);
	
				if($use eq '1' || $use eq '2'){
					$sql = 'UPDATE data SET network_use = ' . $d_dbh->quote($use);
					$sql .= ',notes = ' . $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} changed use to ${use}");
					$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
					$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
				}else{
					push @error, "Invalid Network use: $use\n";
				}
		}elsif(($update eq 'Change CIRCUIT_ID') &&
		   (defined(my $circuit_id=$q->param('circuit_id')))){
				my $sqlnotes = $misc->get_notes($d_dbh,$netid);
				
				$sql = 'UPDATE data SET circuit_id = ' . $d_dbh->quote($circuit_id);
				$sql .= ',notes = ' . $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} changed circuit_id to ${circuit_id}");
				$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
		}elsif(($update eq 'Change Area') &&
		   (defined(my $area=$q->param('area')))){
			if(exists($areas{id}{$area})){
				my $sqlnotes = $misc->get_notes($d_dbh,$netid);

				$sql = 'UPDATE data SET area = ' . $d_dbh->quote($area);
				$sql .= ',notes = ' . $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} changed area to $areas{id}{$area}");
				$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
			}else{
				push @error, "Invalid Area ID: $area";
			}
		}elsif(($update eq 'Change Network') &&
		   (defined(my $network=$q->param('network'))) && (my $cidr=$q->param('cidr'))){
			if($access >= 5){
				if($cidr =~ /^\d{1,2}$/ && ($cidr <= 30 || $cidr >= 24)){
					if($network =~ /^((25[0-5]|2[0-4]\d|1\d{1,2}|\d{1,2}|0)\.){3}(25[0-5]|2[0-4]\d|1\d{1,2}|\d{1,2}|0)$/){
						my $oct4 = $3;
						my $num = $ipmath->cidr_check($cidr);

						if($oct4 % $num){
							push @error, "invalid network specified (${network}/${cidr})";
						}
					}else{
						unless($network eq ''){
							push @error, "invalid network specified (${network}/${cidr})";
						}
					}

					unless(@error){
						my $sqlnotes = $misc->get_notes($d_dbh,$netid);

						$sql = 'UPDATE data SET netblock = ';
						if($network eq ''){
							$sql .= 'NULL, parent_id = NULL';
						}else{
							my $pid = $misc->get_pid($d_dbh,$ipmath,$network,$cidr);
							if($pid == 0){
								if($network =~ /^64\.115\./){
									$pid = 289;
								#}elsif($network =~ /^64\.61\.[0-127]/){
								#	$pid = 276;
								#}elsif($network =~ /^64\.61\./){
								#	$pid = 288;
								}else{
									die "Confused\n";
								}
							}
							
 							$sql .= $d_dbh->quote($network);
							$sql .= ', parent_id = ' . $d_dbh->quote($pid);
						}
						$sql .= ', cidr = ' . $d_dbh->quote($cidr) . ', notes = ';
						$sql .= $d_dbh->quote("${sqlnotes}\n\n${ts}: ${username} network change: ${network}/${cidr}");
						$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

						$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}";
					}
				}else{
			 		push @error, "invalid CIDR specified (/${cidr})";
				}
			}else{
				push @error, "You do not have sufficient permissions to perform this action (network)";
			}
		}elsif(($update eq 'Delete') || ($update eq 'Revoke')){
			if($access >= 4){
				$sql = 'SELECT netblock,cidr,status FROM data WHERE netid = ' . $d_dbh->quote($netid);
				my $sth = $d_dbh->prepare($sql);
				$sth->execute;
				my $row = $sth->fetchrow_arrayref;

			   my $template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		
				my $code = 1; # incase netblock has not been assigned
				if($row->[0]){
					$code = $misc->check_igp($ipmath,$cfg{'igp_host'},$cfg{'igp_community'},$row->[0],$row->[1],$cfg{'igp_ignore_netmask'});
				}
				if($code){
					$sql = 'UPDATE data SET status = 7, ostatus = ' . $d_dbh->quote($row->[2]);
					if($update eq 'Delete'){
						my $sqla = 'SELECT * FROM data WHERE netid = ' . $d_dbh->quote($netid);
						my $stha = $d_dbh->prepare($sqla);
						$stha->execute;
						my $row = $stha->fetchrow_arrayref;
						
						my $sqlb = 'INSERT INTO history VALUES (';
						foreach my $val (@{$row}){
							$sqlb .= $d_dbh->quote($val) . ',';
						}
						chop($sqlb); # trailing comma
						$sqlb .= ')';
						my $sthb = $d_dbh->prepare($sqlb);
						$sthb->execute;
						
						$sql = 'DELETE FROM data';
						my ($oct1,$oct2,$oct3,$oct4) = split /\./, $row->[0];
						my $arpa = "/var/named/${oct4}.${oct3}.${oct2}.${oct1}.in-addr.arpa";
						$template->param(ARPA => $arpa) if(-f $arpa);
						$template->param(VIEW_DELETE_RESULT => 1);
						$template->param(NETID => $netid);
						$template->param(NETBLOCK => $row->[5]);
						$template->param(CIDR => $row->[6]);
					}else{
						if($row->[0]){
							my $time = $^T;
							$sql .= ', last_updated = ' . $d_dbh->quote($time);
							my $db_notes = $misc->get_notes($d_dbh,$netid);
							my $new_notes = "${db_notes}\n${ts}: ${update}d by ${username}\n";
							$sql .= ',notes = ' . $d_dbh->quote($new_notes);
						}else{
							push @error, "Can't revoke an empty network.  Delete instead.";
						}
					}
					$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
					$sth = $d_dbh->prepare($sql);
					$sth->execute;
					$sql = undef; # se we dont fall though
	
					if($update eq 'Revoke'){
						$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}"
					}else{
	
						$template->param(OUR_COMPANY => $cfg{our_name});
						$template->param(SCRIPT => $script);
						$template->param(IPMANAGE => $ipmanage);
						$template->param(IPMONITOR => $ipmonitor);
						$template->param(VERSION => $cfg{version});
						$template->param(USERNAME => $username);
						print $template->output;
						exit;
					}
				}else{
					push @error, "Will not $update network - $row->[0]/$row->[1] found in IGP.";
				}
			} else {
				push @error, "You do not have sufficient permissions to perform this action (unallocate)";
			}
		}elsif(($update eq 'Change Status') &&
		       (defined(my $newstat=$q->param('status'))) && (defined(my $ostat=$q->param('ostatus')))){
			if($access >= 4){
				if($newstat == 7){
					my $sqla = 'SELECT netblock FROM data WHERE netid = ' . $d_dbh->quote($netid);
					my $sth = $d_dbh->prepare($sqla);
					$sth->execute;
					my $row = $sth->fetchrow_arrayref;
					unless($row->[0]){
						push @error, "Cannot revoke empty network.  Delete instead.";
					}
				}elsif($newstat == 2){
					# waiting on allocation - otherwise would allow a bypass of the approval process by access >= 5
					unless($access >= 5){
						push @error, "You do not have sufficient permissions to perform this action (change status-2)";
					}
				}
				my $db_notes = $misc->get_notes($d_dbh,$netid);
				my $newnotes = "${db_notes}\n${ts}: status changed from \'$status{$ostat}\'";
				$newnotes .= " to \'$status{$newstat}\' by ${username}\n";

				$sql = 'UPDATE data SET status = ' . $d_dbh->quote($newstat);
				$sql .= ', ostatus = ' . $d_dbh->quote($ostat);
				$sql .= ', notes = ' . $d_dbh->quote($newnotes);
	 			$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);

				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?view_network=${netid}"
			}else{
				push @error, "You do not have sufficient permissions to perform this action (change status)";
			}
		}elsif ($aty == 1){
			if ($access >= 5){
				my $db_notes = $misc->get_notes($d_dbh,$netid);
				$sql = 'UPDATE data SET status = ';
				$mail_template = "$cfg{home_dir}/data/templates/ipanalyze/text/";
				if($update eq 'Approve'){
					my $newnotes = "$db_notes\n\n$ts request approved by $username";
					if(defined($notes) && ($notes ne '')){
						$newnotes .= " with the following comments:\n" .
										 "$notes";
					}
					$newnotes .= "\n";

					$sql .= '2, notes = ' . $d_dbh->quote($newnotes);
					$mail_template .= 'request_s2.txt';
				}elsif ($update eq "Deny"){
					my $newnotes = "$db_notes\n\n$ts request denied by $username";
					if(defined($notes) && ($notes ne '')){
						$newnotes .= " with the following comments:\n" .
										 "$notes";
					}
					$newnotes .= "\n";

					$sql .= '6, notes = ' . $d_dbh->quote($newnotes);
					$mail_template .= 'request_deny.txt';
				}
				$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?action=view_details&cust_id=${cust_id}";

			} else {
				push @error, "You do not have sufficient permissions to perform this action (and)";
			}
		}elsif ($aty == 2){
			# allocate
			if ($access >= 4){
				my $netblock = $q->param('allocate_ip');
				my $cidr = $q->param('allocate_slash');
				# below for in case someone entered the block manually (if none were available, etc)
				my $parent_id = $misc->get_pid($d_dbh,$ipmath,$netblock,$cidr);
				unless($parent_id eq $q->param('parent_id')){
					warn 'reported parent_id: ', $q->param('parent_id'), " - actual: $parent_id\n";
				}

				die "invalid parent_id: $parent_id\n" unless($parent_id == int($parent_id));

				if($netblock !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ || $cidr !~ /^\d{1,2}$/){
					push @error , "Invalid IP address entered";
				} else {
					#Check for a vaild netblock/CIDR
					my $net = $ipmath->get_netblock($netblock,$cidr);
					unless($net eq $netblock){
						push @error , "${netblock}/${cidr} inconsistent subnet";
					}
				
					my $sqla = 'SELECT circuit_id FROM data WHERE netblock = ' . $d_dbh->quote($netblock);
					my $stha = $d_dbh->prepare($sqla);
					$stha->execute;
					my $row = $stha->fetchrow_arrayref;
					my $circuit_id = $row->[0];
					if(defined($circuit_id)){
						push @error , "$netblock is already in the database assigned to circuitid $circuit_id";
					} else {
						my $db_notes = $misc->get_notes($d_dbh,$netid);
						my $newnotes = "$db_notes\n\n$ts Allocated by by $username";
						if(defined($notes) && ($notes ne '')){
							$newnotes .= " with the following comments:\n" .
							             "$notes";
						}	
						$newnotes .= "\n";
						$sql = 'UPDATE data SET status = 3, parent_id = ' . $d_dbh->quote($parent_id);
						$sql .= ',netblock = ' . $d_dbh->quote($netblock);
						$sql .= ',cidr = ' . $d_dbh->quote($cidr);
						$sql .= ',notes = ' . $d_dbh->quote($newnotes);
						$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
						$mail_template .= 'request_allocate.txt';

						$misc->loggit($netid,$netblock,$cidr); # log to /var/tmp/ipanalyze.log (LOG)
					}
				}
				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?action=view_details&cust_id=${cust_id}";
			}else {
				push @error, "You do not have sufficient permissions to perform this action (allocate)";
			}
		}elsif ($aty == 3){ 
			# assign networks
			if ($access > 2){
				my $db_notes = $misc->get_notes($d_dbh,$netid);
				if ($update eq 'Assign'){
					my $newnotes = "$db_notes\n\n$ts request closed by by $username";
					if(defined($notes) && ($notes ne '')){
						$newnotes .= " with the following comments:\n" .
						             "$notes";
					}	

					$sql = 'UPDATE data SET status = 0, enable_monitoring = 1, enable_emailwar = 1';
					$sql .= ', notes = ' . $d_dbh->quote($newnotes);
					$sql .= ' WHERE netid = ' . $d_dbh->quote($netid);
					$mail_template .= 'request_complete.txt';
				}
				$link = "http://$ENV{'HTTP_HOST'}$ENV{'SCRIPT_NAME'}?action=view_details&cust_id=${cust_id}";
			} else {
				push @error, "You do not have sufficient permissions to perform this action (assign)";
			}
		}
	}
	if (@error){
		my $template = HTML::Template->new(filename => "$cfg{html_dir}/error.html");
		my @loop_data;
		foreach my $error (@error){
			my %row_data = ( ERROR_DATA => $error );
			push @loop_data, \%row_data;
		}
		$template->param(ERROR_LOOP => \@loop_data);
		print $template->output;
	} else {
		if($sql){
			warn "default execute: $sql\n";
			my $sth = $d_dbh->prepare($sql);
			$sth->execute();
		}
		if(-f $mail_template){
			$misc->mail_update($d_dbh,$c_dbh,$customers,$netid,$mail_template,\%cfg);
		}
		#print $q->redirect($link);
		print '<meta http-equiv="refresh" content="0; url=' . $link . '">';
	}
	exit;
}

sub show_main {
	my $d_dbh = shift;
	my $username = shift;
	my $q = shift;
	my $type = shift;
	my $vquery = shift;
	my $atype = shift;
	my $access_needed = shift;
	my $access = shift;
	my $template;
	if($type eq 'default'){
		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		if ($access >= $access_needed){
			my ($compl,$wcr,$wal,$was,$hold,$pend,$revok) = $misc->stats($d_dbh,'0:1:2:3:4:6:7');
			
			my $sqle = 'SELECT COUNT(*) FROM data WHERE average_usage = 100';
			my $sthe = $d_dbh->prepare($sqle);
			$sthe->execute;
			my ($pu) = $sthe->fetchrow_array;

			my $sqla = 'SELECT COUNT(*) FROM data WHERE average_usage < 100 AND average_usage > 79';
			my $stha = $d_dbh->prepare($sqla);
			$stha->execute;
			my ($hu) = $stha->fetchrow_array;

			my $sqlb = 'SELECT COUNT(*) FROM data WHERE average_usage <= 79 AND average_usage >= 40';
			my $sthb = $d_dbh->prepare($sqlb);
			$sthb->execute;
			my ($mu) = $sthb->fetchrow_array;

			my $sqlc = 'SELECT COUNT(*) FROM data WHERE average_usage < 40 AND average_usage > 0';
			my $sthc = $d_dbh->prepare($sqlc);
			$sthc->execute;
			my ($lu) = $sthc->fetchrow_array;

			my $sqld = 'SELECT COUNT(*) FROM data WHERE average_usage = 0';
			my $sthd = $d_dbh->prepare($sqld);
			$sthd->execute;
			my ($zu) = $sthd->fetchrow_array;

			$template->param(COMPLETE => $compl);
			$template->param(RWCR => $wcr);
			$template->param(REAL => $wal);
			$template->param(RWA => $was);
			$template->param(HOLD => $hold);
			$template->param(PENDING => $pend);
			$template->param(REVOKED => $revok);
			$template->param(PU => $pu); # perfect usage
			$template->param(HU => $hu);
			$template->param(MU => $mu);
			$template->param(LU => $lu);
			$template->param(ZU => $zu);
			$template->param(STATS => 1);
		}else{
			my @error = ("You do not have sufficient permissions to perform this action (default)");
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
   }elsif($type eq 'view_network'){
		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		$template->param(VIEW_NETWORK => 1);
		my $sql = 'SELECT * FROM data WHERE netid = ' . $d_dbh->quote($vquery);
		my $sth = $d_dbh->prepare($sql);
		$sth->execute;
		my $row=$sth->fetchrow_hashref;

		my $view_history = $misc->update_history($d_dbh,$vquery,$username);
		my $status = $customers->status($c_dbh,$d_dbh,$row->{circuit_id},$row->{cust_id});

		$template->param(OC_COLOR => $color{$status} || '#FFFFFF');
		$template->param(CUST_ID => $row->{cust_id});
		$template->param(CIRCUIT_ID => $row->{circuit_id});
		$template->param(OC8_STATUS => $status);
		$template->param(LOCAL_CONTACT => $row->{local_contact});
		$template->param(LOCAL_EMAIL => $row->{local_email});
		$template->param(TECH_CONTACT => $row->{tech_name});
		$template->param(TECH_CONTACT_PHONE => $row->{tech_phone});
		$template->param(TECH_EXT => $row->{tech_ext}) if(defined($row->{tech_ext}));
		$template->param(TECH_CONTACT_EMAIL => $row->{tech_email});
		$template->param(PRIVATE => (defined($row->{private_id})) ? 'Yes' : 'No');
		$template->param(NETID => $vquery);
		$template->param(AREA => $areas{id}{$row->{area}});
		$template->param(NETBLOCK => $row->{netblock});
		$template->param(NETWORK_USE => ($row->{network_use} eq '1') ? 'WAN' : 'LAN');
		$template->param(CIDR => $row->{cidr});
		$template->param(STATUS => $status{$row->{status}});
		my $date = localtime($row->{bornondate});
		$template->param(DATE => $date);
 		$template->param(NOTES => $row->{notes});
		$template->param(VIEW_HISTORY => $view_history);
		$template->param(ENABLE_MONITORING => ($row->{enable_monitoring}) ? 'Yes' : 'No');
		$template->param(ENABLE_EWARN => ($row->{enable_emailwar}) ? 'Yes' : 'No');
		if($row->{status} eq '0'){
			$template->param(SHOW_USAGE => 1);
			$template->param(LUSAGE => $row->{current_usage});
			$template->param(AVRG => $row->{average_usage});
		}
		if($access >= 4){
			$template->param(SHOW_ADMIN => 1);
			$template->param(ALLOW_REVOKE => 1) if($row->{status} == 0 || $row->{status} == 3);
		}
	}elsif($type eq 'show_search'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			$template->param(SHOW_SEARCH => 1);
			my @data;
			while(my($key,$value) = each %status){
				my %hash;
				$hash{TID} = $key;
				$hash{TNAME} = $value;
				push @data, \%hash;
			}
			$template->param(STATUSES => \@data);
		} else {
			my @error;
			push @error, "You do not have sufficient permissions to perform this action (show_searc)";
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'view_list_data'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			my $stuff = 'netid,cust_id,netblock,cidr,circuit_id,bornondate';
			my $sql = "SELECT $stuff FROM data WHERE status = " . $d_dbh->quote($vquery);
			$sql .= ' ORDER BY bornondate DESC';
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my @request_data;
			my $n=0;
			my (%cname,%cstatus);
			while(my $data = $sth->fetchrow_arrayref){
				$n++;
				my $entry = localtime($data->[5]);
				if($cname{$data->[1]} && (! $cstatus{$data->[4]})){
					$cstatus{$data->[4]} = $customers->status($c_dbh,$d_dbh,$data->[4],$data->[1]);
				}elsif($cstatus{$data->[4]} && (! $cname{$data->[1]})){
					$cname{$data->[1]} = $customers->name($c_dbh,$d_dbh,'cust_id',$data->[1]);
				}elsif((! $cname{$data->[1]}) && (! $cstatus{$data->[4]})){
					my $all = $customers->all($c_dbh,$d_dbh,'circuit_id',$data->[4]);
					$cname{$data->[1]} = $all->[1];
					$cstatus{$data->[4]} = $all->[6];
				}

				push @request_data, {
				  BGCOLOR => $color{$cstatus{$data->[4]}},
				  COMPANY_NAME => $cname{$data->[1]},
				  NETWORK => $data->[2],
				  SLASH => $data->[3],
				  CIRCUIT_ID => $data->[4],
				  OC8_STATUS => $cstatus{$data->[4]},
				  NETID => $data->[0],
				  DATE => $entry,
				  CUST_ID => $data->[1],
				};
			}
			$template->param(VIEW_LIST_DATA => 1);
			if($n > 0){
				if($q->param('sort')){
					my @sorted = sort { $a->{$q->param('sort')} cmp $b->{$q->param('sort')} } @request_data;
					$template->param(REQUEST_DATA => \@sorted);
				}else{
					$template->param(REQUEST_DATA => \@request_data);
				}
			}else{
				$template->param(NO_RESULTS_FOUND => 1);
			}
		} else {
			my @error;
			push @error, "You do not have sufficient permissions to perform this action (view_list_data)";
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'view_allocation'){
		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		if(my $what=$q->param('what')){
			if($what eq 'all'){
				my $sql = 'SELECT id,cidr,area,allow_use FROM parent_networks';
				my $sth = $d_dbh->prepare($sql);
				$sth->execute;
				my %total;
				while(my $row=$sth->fetchrow_arrayref){
					my $id = $row->[0];
					my $cidr = $row->[1];
					my $area = $row->[2];
					my $allow_use = $row->[3];

					if($allow_use){
						$total{'available'}{$area} += $ipmath->cidr_check($cidr);
					}else{
						$total{'prohibited'}{$area} += $ipmath->cidr_check($cidr);

						my $sqla = 'SELECT cidr FROM data WHERE parent_id = ' . $d_dbh->quote($id);
						my $stha = $d_dbh->prepare($sqla);
						$stha->execute;
						while(my $rowa=$stha->fetchrow_arrayref){
							$total{'prohibited'}{$area} -= $ipmath->cidr_check($rowa->[0]);
						}
					}
				}

				my %used;
				foreach my $area (sort keys %{$areas{id}}){
					foreach(19..30){
						$used{$area}{$_} = 0;
					}
				}
				$sql = 'SELECT parent_networks.area,data.cidr FROM data,parent_networks WHERE data.netblock IS NOT NULL';
				$sql .= ' AND data.parent_id = parent_networks.id AND parent_networks.allow_use = 1';
				$sth = $d_dbh->prepare($sql);
				$sth->execute;
				while(my $row=$sth->fetchrow_arrayref){
					$total{'allocated'}{$row->[0]} += $ipmath->cidr_check($row->[1]);
					$used{$row->[0]}{$row->[1]}++;
				}

				my @data;
				foreach my $area (sort keys %{$areas{id}}){
					my %info;
					$info{'AREA'} = $areas{id}{$area};
					my $delta = ($total{'available'}{$area} - $total{'allocated'}{$area});
					if($total{'available'}{$area}){
						$info{'PERCENT_USED'} = sprintf("%.2f",(100-($delta/$total{'available'}{$area}*100)));
					}else{
						$info{'PERCENT_USED'} = 100;
					}
					$info{'TOTAL_IPS'} = $total{'available'}{$area} || 0;
					$info{'IPS_ALLOCATED'} = $total{'allocated'}{$area} || 0;
					$info{'PROHIBITED_IPS'} = $total{'prohibited'}{$area} || 0;
					foreach my $cidr (keys %{$used{$area}}){
						$info{"USED_${cidr}"} = $used{$area}{$cidr} || 0;
					}
					push @data, \%info;
				}

				$template->param(SHOW_ALLOCATION => 1);
				$template->param(REQUEST_DATA => \@data);
				$template->param(GRAPH => "/cgi-bin/graph.mpl?utilizations=1");
			}else{
				die "confused";
			}
		}else{
			die "confused";
		}
	}elsif($type eq 'view_cidr'){
		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		$template->param(VIEW_CIDR => 1);
		my $sql = 'SELECT cidr FROM data WHERE netblock IS NOT NULL';
		my $sth = $d_dbh->prepare($sql);
		$sth->execute;
		my (%cidr,$total,@data,$link,$bgcolor);
		while(my $row = $sth->fetchrow_arrayref){
			$cidr{$row->[0]} ++;
		}
		foreach my $value (values %cidr){
			$total += $value;
		}
		my $n = 30;
		while($n > 19){
			$bgcolor = ($bgcolor) ? '' : 'bgcolor="#D9D9D9"';
				
			my $value = $cidr{$n} || 0;
			my $percent = sprintf("%.2f",($value / $total * 100));
			push @data, { BGCOLOR => $bgcolor, SIZE => $n, VALUE => $value, PERCENT => $percent, };
			$link .= '&' . $n . '=' . $value;
			$n--;
		}
		$template->param(CIDR_DATA => \@data);
		$template->param(GRAPH => "/cgi-bin/graph.mpl?cidr=1&${link}");

	}elsif($type eq 'view_usage'){
		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		$template->param(VIEW_USAGE => 1);
		if(my $what=$q->param('what')){
			my $sql = 'SELECT netid,cust_id,circuit_id,netblock,cidr,usage_history,status FROM data';
			$sql .= ' WHERE enable_monitoring = 1 AND status = 0 ORDER BY cidr ASC';
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my @data;
			my(%cname,%cstatus);
			while(my $row=$sth->fetchrow_arrayref){
				if($row->[5]){
					my $last_poll = (split /:/, $row->[5])[-1];

					if( ($what eq 'high' && $last_poll >= 80) ||
					    ($what eq 'medium' && $last_poll >= 40 && $last_poll <= 79) ||
					    ($what eq 'low' && $last_poll > 0 && $last_poll <= 39) ||
					    ($what eq 'zero' && $last_poll == 0 && defined($last_poll)) ){

						my $network = $row->[3] . '/' . $row->[4];

						if($cname{$row->[1]} && (! $cstatus{$row->[2]})){
							$cstatus{$row->[2]} = $customers->status($c_dbh,$d_dbh,$row->[2],$row->[1]);
						}elsif($cstatus{$row->[2]} && (! $cname{$row->[1]})){
							$cname{$row->[1]} = $customers->name($c_dbh,$d_dbh,'cust_id',$row->[1]);
						}elsif((! $cname{$row->[1]}) && (! $cstatus{$row->[2]})){
							my $all = $customers->all($c_dbh,$d_dbh,'circuit_id',$row->[2]);
							$cname{$row->[1]} = $all->[1];
							$cstatus{$row->[2]} = $all->[6];
						}

						push @data, {
						  BGCOLOR => $color{$cstatus{$row->[2]}},
						  COMPANY_NAME => $cname{$row->[2]},
						  OC8_STATUS => $cstatus{$row->[2]},
						  CUST_ID => $row->[1],
						  NETID => $row->[0],
						  CIRCUIT_ID => $row->[2],
						  NETWORK => $row->[3] . '/' . $row->[4],
						  STATUS => $status{$row->[6]},
						};
					}
				}
			}
			if($what eq 'utilization'){
				$template->param(GRAPH => '/cgi-bin/graph.mpl?low=0&high=100&all=1');
			}elsif($what eq 'high'){
				$template->param(GRAPH => '/cgi-bin/graph.mpl?low=80&high=100');
				$template->param(VIEW_SEARCH_RESULT => 1);
				$template->param(REQUEST_DATA => \@data);
			}elsif($what eq 'medium'){
				$template->param(GRAPH => '/cgi-bin/graph.mpl?low=40&high=79');
				$template->param(VIEW_SEARCH_RESULT => 1);
				$template->param(REQUEST_DATA => \@data);
			}elsif($what eq 'low'){
				$template->param(GRAPH => '/cgi-bin/graph.mpl?low=1&high=39');
				$template->param(VIEW_SEARCH_RESULT => 1);
				$template->param(REQUEST_DATA => \@data);
			}elsif($what eq 'zero'){
				$template->param(GRAPH => '/cgi-bin/graph.mpl?low=0&high=0');
				$template->param(VIEW_SEARCH_RESULT => 1);
				$template->param(REQUEST_DATA => \@data);
			}else{
				die "confused";
			}
		}else{
			my $sql = 'SELECT cust_id,netblock,cidr,circuit_id,usage_history FROM data';
			$sql .= ' WHERE netid = ' . $d_dbh->quote($vquery);
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $row = $sth->fetchrow_arrayref;

			$template->param(COMPANY => $customers->name($c_dbh,$d_dbh,'cust_id',$row->[0]));
			$template->param(NETID => $vquery);
			$template->param(NETBLOCK => $row->[1]);
			$template->param(CIDR => $row->[2]);
			$template->param(CIRCUIT_ID => $row->[3]);
			$template->param(GRAPH => "/cgi-bin/graph.mpl?netid=${vquery}");
		}
	}elsif($type eq 'view_details'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			$template->param(ATY => $atype);
			$template->param(VIEW_DETAIL => 1);
			my $stuff = 'netid,circuit_id,area,netblock,cidr,network_use,status,bornondate,current_usage,average_usage';
			my $sql = "SELECT $stuff FROM data WHERE cust_id = " . $d_dbh->quote($vquery);
			$sql .= ' ORDER BY netid DESC';
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my @data;
			while (my $row=$sth->fetchrow_hashref){
				$misc->update_history($d_dbh,$row->{netid},$username);
				my $time = localtime($row->{bornondate});
				my %data = (NETID => $row->{netid},
				            CIRCUIT_ID => $row->{circuit_id},
				            AREA => $areas{id}{$row->{area}},
				            NETBLOCK => $row->{netblock},
				            CIDR => $row->{cidr},
				            NETWORK_USE => ($row->{network_use} == 1) ? 'WAN' : 'LAN',
				            STATUS => $status{$row->{status}},
				            DATE => $time,
				            CURR_USAGE => defined($row->{current_usage}) ? $row->{current_usage} : 'N/A',
				            AVRG => defined($row->{average_usage}) ? $row->{average_usage} : 'N/A',
				           );
				
				push @data, \%data;
			}
			$sth->finish;
			$template->param(NETWORK_INFO => \@data);

			my $cust = $customers->all($c_dbh,$d_dbh,'cust_id',$vquery);
			my $country = $cust->[7] || 'US';

			$template->param(COMPANY_NAME => $cust->[1]);
			$template->param(CUST_ID => $vquery);
			$template->param(ADDRESS => $cust->[2]);
			$template->param(CITY => $cust->[3]);
			$template->param(STATE => $cust->[4]);
			$template->param(ZIP => $cust->[5]);
			$template->param(COUNTRY => $country);
		} else {
			my @error = ("You do not have sufficient permissions to perform this action (view_detail)");
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'edit_contact'){
		my @error;
		if ($access >= $access_needed){
			push @error, "Modify Customer contact info in Open Cafe.";
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");

			my $cust = $customers->all($c_dbh,$d_dbh,'cust_id',$vquery);
			my $country = $cust->[7] || 'US';

			$template->param(COMPANY_NAME => $cust->[1]);
			$template->param(CUST_ID => $vquery);
			$template->param(ADDRESS => $cust->[2]);
			$template->param(CITY => $cust->[3]);
			$template->param(STATE => $cust->[4]);
			$template->param(ZIP => $cust->[5]);
			$template->param(COUNTRY => $country);

			$template->param(ATY => $atype);
			$template->param(UPDATE_TYPE => 'contact');
			$template->param(EDIT_CONTACT => 1);
		} else {
			push @error, "You do not have sufficient permissions to perform this action (edit_contact)";
		}
		$misc->show_error($cfg{html_dir},\@error);
		exit;
	}elsif($type eq 'edit_in_contact'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			my $sql = 'SELECT local_contact,local_email FROM data WHERE netid = ' . $d_dbh->quote($vquery);
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $data = $sth->fetchrow_arrayref;
			$template->param(NETID => $vquery);
			$template->param(ATY => $atype);
			$template->param(LOCAL_CONTACT => $data->[0]);
			$template->param(LOCAL_EMAIL => $data->[1]);
			$template->param(UPDATE_TYPE => 'in_contact');
			$template->param(EDIT_IN_CONTACT => 1);
		} else {
			my @error;
			push @error, "You do not have sufficient permissions to perform this action (edit_in_contact)";
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'edit_tech_contact'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			my $stuff = 'tech_name,tech_email,tech_phone,tech_ext,private_id,status';
			my $sql = "SELECT $stuff FROM data WHERE netid = " . $d_dbh->quote($vquery);
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $data = $sth->fetchrow_arrayref;
			$template->param(NETID => $vquery);
			$template->param(ATY => $atype);
			$template->param(TECH_NAME => $data->[0]);
			$template->param(TECH_EMAIL => $data->[1]);
			$template->param(TECH_PHONE => $data->[2]);
			$template->param(TECH_EXT => $data->[3]) if(defined($data->[3]));
			my $what = (defined($data->[4])) ? 'YES' : 'NO';
			$template->param("PRIVATE_$what" => 'selected');
			$template->param(UPDATE_TYPE => 'tech_contact');
			$template->param(EDIT_TECH_CONTACT => 1);
			$template->param(HOLDING_ON => 1) if($data->[5] == 4);
		} else {
			my @error;
			push @error, "You do not have sufficient permissions to perform this action (edit_tech_contact)";
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'edit_request'){
		if ($access >= $access_needed){
			$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
			my $stuff = 'cust_id,circuit_id,area,netblock,cidr,network_use,status,ostatus,bornondate';
			my $sql = "SELECT $stuff FROM data WHERE netid = " . $d_dbh->quote($vquery);
			my $sth = $d_dbh->prepare($sql);
			$sth->execute;
			my $data = $sth->fetchrow_hashref;
			my $area = $data->{area};
			my $status = $data->{status};
			$template->param(CUST_ID => $data->{cust_id});
			$template->param(NETID => $vquery);
			$template->param(ATY => $status);
			#What to display in the request update form.
			if($status == 1){
				$template->param(AND => 'and');
			}elsif($status == 2){
				$template->param(ALLOCATE => 'allocate');
				my $cidr = $data->{cidr};

				
				my ($netblock,$parent_id) = $ipmath->get_network($d_dbh,$cidr,$area);
				if($netblock){
					my @error;
					if($q->param('override')){
						unless($access >= 5){
							push @error, "override received, but I don't respect your authority.\n";
						}
					}else{
						my $code = $misc->check_igp($ipmath,$cfg{'igp_host'},$cfg{'igp_community'},$netblock,$cidr,$cfg{'igp_ignore_netmask'});
						if($code eq '0'){
							warn "PROBLEM with $netblock/$cidr\n";
							push @error, "${netblock}/${cidr} is fully or partially used according to IGP.";
						}
					}
					if(@error){
						$misc->show_error($cfg{html_dir},\@error);
						exit;
					}
					if($access >= 4){
						$template->param(PARENT_ID => $parent_id);
						$template->param(NETBLOCK => $netblock);
						$template->param(CIDR => $cidr);
						$template->param("SLASH_${cidr}" => 'selected');
					}
				}else{
					$template->param(NONEAVAIL => 1);
				}
			}elsif($status == 3){
				$template->param(ASSIGN => 'assign');
			}elsif($status == 4){
				$template->param(HOLD => 'hold');
			}elsif($status == 7){
				$template->param(REVOKED_CODE => 8);
				$template->param(OSTATUS_CODE => $data->{ostatus});
				$template->param(OSTATUS => $status{$data->{ostatus}});
			}
		
			unless($status == 7){
				my @statuses;
				foreach my $key (keys %status){
					my %data;
					$data{CODE} = $key;
					$data{NAME} = $status{$key};
					$data{SELECTED} = 'selected' if($data->{status} == $key);
					push @statuses, \%data;
				}
				$template->param(STATUSES => \@statuses);
				$template->param(OSTATUS => $data->{status});
			}
			if((($data->{status} == 0) && defined($data->{status})) || $data->{status} == 3){
				if($access >= 4){
					$template->param(ALLOW_EDIT => 1);
					my $cidrparam = 'SLASH_' . $data->{cidr};
					$template->param($cidrparam => 'selected');
				}
			}
			my @areas;
			while(my($key,$value) = each %{$areas{id}}){
				my %data;
				$data{CODE} = $key;
				$data{NAME} = $value;
				$data{SELECTED} = 'selected' if($area == $key);
				push @areas, \%data;
			}
			$template->param(CIRCUIT_ID => $data->{circuit_id});
			$template->param(AREAS => \@areas);
			$template->param(IPADDRESS => $data->{netblock});
			$template->param(SLASH => $data->{cidr});
			my $use = ($data->{network_use} eq '1') ? 'W' : 'L';
			$use .= 'AN_SELECTED';
			$template->param($use => 'selected');
			$template->param(STATUS => $status{$data->{status}});
			my $time = localtime($data->{bornondate});
			$template->param(BORNONDATE => $time);
			#$template->param(NOTES => $data->{notes}); # dont show old notes
			$template->param(UPDATE_TYPE => 'request_info');
			$template->param(EDIT_REQUEST_INFO => 1);
		}else{
			my @error;
			push @error, "You do not have sufficient permissions to perform this action (edit_request)";
			$misc->show_error($cfg{html_dir},\@error);
			exit;
		}
	}elsif($type eq 'edit_monitoring'){
		my $sql = 'SELECT cust_id,circuit_id,enable_monitoring,enable_emailwar FROM data';
		$sql .= ' WHERE netid = ' . $d_dbh->quote($vquery);
		my $sth = $d_dbh->prepare($sql);
		$sth->execute;
		my $data = $sth->fetchrow_arrayref;

		$template = HTML::Template->new(filename => "$cfg{html_dir}/main.html");
		$template->param(NETID => $vquery);
		$template->param(COMPANY_NAME => $customers->name($c_dbh,$d_dbh,'cust_id',$data->[0]));
		$template->param(CIRCUIT_ID => $data->[1]);
		$template->param(ATY => $atype);
		$template->param(UPDATE_TYPE => 'monitoring_info');
		$template->param(EDIT_MONITORING_INFO => 1);

		
		my $eparam = ($data->[2]) ? 'YES' : 'NO';
		$template->param("MONITORING_$eparam" => 'selected');

		my $wparam = ($data->[3]) ? 'YES' : 'NO';
		$template->param("WARNING_$wparam" => 'selected');
	}else{
		die "confuzzled";
	}
	
	$template->param(SCRIPT => $script);
	$template->param(LOGIN => $login);
	$template->param(IPMANAGE => $ipmanage);
	$template->param(IPMONITOR => $ipmonitor);
	$template->param(SHOW_HOUSTON => 1);
	$template->param(VERSION => $cfg{version});
	$template->param(USERNAME => $username);
	$template->param(ACTION => $q->param('action'));
	$template->param(OUR_COMPANY => $cfg{'our_name'});
	print $template->output;

}
